ET.Penman2 <- 
function (data, constants, ts = "daily", solar = "sunshine hours", 
          wind = "yes", windfunction_ver = 1948, alpha = 0.08, z0 = 0.001, 
          message = "yes", save.csv = "yes", ...) 
{
  if (is.null(data$Tmax) | is.null(data$Tmin)) {
    stop("Required data missing for 'Tmax' and 'Tmin', or 'Temp'")
  }
  if (is.null(data$va) | is.null(data$vs)) {
    if (is.null(data$RHmax) | is.null(data$RHmin)) {
      stop("Required data missing: need either 'va' and 'vs', or 'RHmax' and 'RHmin' (or 'RH')")
    }
  }
  if (wind == "yes") {
    if (is.null(data$uz) & is.null(data$u2)) {
      stop("Required data missing for 'uz' or 'u2'")
    }
  }
  if (solar == "data" & is.null(data$Rs)) {
    stop("Required data missing for 'Rs'")
  }  else if (solar == "sunshine hours" & is.null(data$n)) {
    stop("Required data missing for 'n'")
  }  else if (solar == "cloud" & is.null(data$Cd)) {
    stop("Required data missing for 'Cd'")
  }  else if (solar == "monthly precipitation" & is.null(data$Precip)) {
    stop("Required data missing for 'Precip'")
  }
  if (wind != "yes" & wind != "no") {
    stop("Please choose if actual data will be used for wind speed from wind = 'yes' and wind = 'no'")
  }
  if (wind == "yes") {
    if (is.na(as.numeric(alpha))) {
      stop("Please use a numeric value for the alpha (albedo of evaporative surface)")
    }
    if (!is.na(as.numeric(alpha))) {
      if (as.numeric(alpha) < 0 | as.numeric(alpha) > 1) {
        stop("Please use a value between 0 and 1 for the alpha (albedo of evaporative surface)")
      }
    }
    if (is.na(as.numeric(z0))) {
      stop("Please use a numeric value for the z0 (roughness height)")
    }
  }
  Ta <- (data$Tmax + data$Tmin)/2
  P <- 101.3 * ((293 - 0.0065 * constants$Elev)/293)^5.26
  delta <- 4098 * (0.6108 * exp((17.27 * Ta)/(Ta + 237.3)))/((Ta + 
                                                                237.3)^2)
  gamma <- 0.00163 * P/constants$lambda
  d_r2 <- 1 + 0.033 * cos(2 * pi/365 * data$J)
  delta2 <- 0.409 * sin(2 * pi/365 * data$J - 1.39)
  w_s <- acos(-tan(constants$lat_rad) * tan(delta2))
  N <- 24/pi * w_s
  R_a <- (1440/pi) * d_r2 * constants$Gsc * (w_s * sin(constants$lat_rad) * 
                                               sin(delta2) + cos(constants$lat_rad) * cos(delta2) * 
                                               sin(w_s))
  R_so <- (0.75 + (2 * 10^-5) * constants$Elev) * R_a
  if (solar == "data") {
    R_s <- data$Rs
  }  else if (solar == "monthly precipitation") {
    R_s <- (constants$as + constants$bs * (data$n/N)) * R_a
  }  else {
    R_s <- (0.85 - 0.047 * data$Cd) * R_a
  }
  if (wind == "yes") {
    if (is.null(data$u2)) {
      u2 <- data$uz * log(2/z0)/log(constants$z/z0)
    }
    else {
      u2 <- data$u2
    }
    if (!is.null(data$va) & !is.null(data$vs)) {
      vabar <- data$va
      vas <- data$vs
    }
    else {
      vs_Tmax <- 0.6108 * exp(17.27 * data$Tmax/(data$Tmax + 
                                                   237.3))
      vs_Tmin <- 0.6108 * exp(17.27 * data$Tmin/(data$Tmin + 
                                                   237.3))
      vas <- (vs_Tmax + vs_Tmin)/2
      vabar <- (vs_Tmin * data$RHmax/100 + vs_Tmax * data$RHmin/100)/2
    }
    R_nl <- constants$sigma * (0.34 - 0.14 * sqrt(vabar)) * 
      ((data$Tmax + 273.2)^4 + (data$Tmin + 273.2)^4)/2 * 
      (1.35 * R_s/R_so - 0.35)
    R_ns <- (1 - alpha) * R_s
    R_n = R_ns - R_nl
    if (windfunction_ver == "1948") {
      f_u = 2.626 + 1.381 * u2
    }
    else if (windfunction_ver == "1956") {
      f_u = 1.313 + 1.381 * u2
    }
    else if (windfunction_ver != "1948" & windfunction_ver != 
             "1956") {
      stop("Please select the version of wind function (1948 or 1956)")
    }
    Ea = f_u * (vas - vabar)
    Epenman.Daily <- delta/(delta + gamma) * (R_n/constants$lambda) + 
      gamma/(delta + gamma) * Ea
  }  else {
    RHmean <- (data$RHmax + data$RHmin)/2
    Epenman.Daily <- 0.047 * R_s * sqrt(Ta + 9.5) - 2.4 * 
      (R_s/R_a)^2 + 0.09 * (Ta + 20) * (1 - RHmean/100)
  }
  ET.Daily <- Epenman.Daily
  ET.Monthly <- aggregate(ET.Daily, as.yearmon(data$Date.daily, 
                                               "%m/%y"), FUN = sum)
  ET.MonthlyAve <- ET.AnnualAve <- NULL
  for (mon in min(as.POSIXlt(data$Date.daily)$mon):max(as.POSIXlt(data$Date.daily)$mon)) {
    i = mon - min(as.POSIXlt(data$Date.daily)$mon) + 1
    ET.MonthlyAve[i] <- mean(ET.Daily[as.POSIXlt(data$Date.daily)$mon == 
                                        mon])
  }
  ET.Annual <- aggregate(ET.Daily, floor(as.numeric(as.yearmon(data$Date.daily, 
                                                               "%m/%y"))), FUN = sum)
  for (year in min(as.POSIXlt(data$Date.daily)$year):max(as.POSIXlt(data$Date.daily)$year)) {
    i = year - min(as.POSIXlt(data$Date.daily)$year) + 1
    ET.AnnualAve[i] <- mean(ET.Daily[as.POSIXlt(data$Date.daily)$year == 
                                       year])
  }
  ET_formulation <- "Penman"
  if (wind == "no") {
    ET_type <- "Open-water Evaporation"
    Surface <- paste("water, albedo =", alpha, "; roughness height =", 
                     z0, "m")
  }  else {
    if (alpha != 0.08) {
      ET_type <- "Potential ET"
      Surface <- paste("user-defined, albedo =", alpha, 
                       "; roughness height =", z0, "m")
    }    else if (alpha == 0.08) {
      ET_type <- "Open-water Evaporation"
      Surface <- paste("water, albedo =", alpha, "; roughness height =", 
                       z0, "m")
    }
  }
  if (solar == "data") {
    message1 <- "Solar radiation data have been used directly for calculating evapotranspiration"
  }  else if (solar == "sunshine hours") {
    message1 <- "Sunshine hour data have been used for calculating incoming solar radiation"
  }  else if (solar == "cloud") {
    message1 <- "Cloudiness data have been used for calculating sunshine hour and thus incoming solar radiation"
  }  else {
    message1 <- "Monthly precipitation data have been used for calculating incoming solar radiation"
  }
  if (wind == "yes") {
    if (windfunction_ver == "1948") {
      message2 <- "Wind data have been used for calculating the Penman evaporation. Penman 1948 wind function has been used."
    }    else if (windfunction_ver == "1956") {
      message2 <- "Wind data have been used for calculating the Penman evaporation. Penman 1956 wind function has been used."
    }
  }  else {
    message2 <- "Alternative calculation for Penman evaporation without wind data have been performed"
  }
  results <- list(ET.Daily = ET.Daily, ET.Monthly = ET.Monthly, 
                  ET.Annual = ET.Annual, ET.MonthlyAve = ET.MonthlyAve, 
                  ET.AnnualAve = ET.AnnualAve, ET_formulation = ET_formulation, 
                  ET_type = ET_type, message1 = message1, message2 = message2)
  if (ts == "daily") {
    res_ts <- ET.Daily
  }  else if (ts == "monthly") {
    res_ts <- ET.Monthly
  }  else if (ts == "annual") {
    res_ts <- ET.Annual
  }
  if (message == "yes") {
    message(ET_formulation, " ", ET_type)
    message("Evaporative surface: ", Surface)
    message(message1)
    message(message2)
    message("Timestep: ", ts)
    message("Units: mm")
    message("Time duration: ", time(res_ts[1]), " to ", time(res_ts[length(res_ts)]))
    if (NA %in% res_ts) {
      message(length(res_ts), " ET estimates obtained; ", 
              length(which(is.na(res_ts))), " NA output entries due to missing data")
      message("Basic stats (NA excluded)")
      message("Mean: ", round(mean(res_ts, na.rm = T), 
                              digits = 2))
      message("Max: ", round(max(res_ts, na.rm = T), digits = 2))
      message("Min: ", round(min(res_ts, na.rm = T), digits = 2))
    }
    else {
      message("Basic stats")
      message("Mean: ", round(mean(res_ts), digits = 2))
      message("Max: ", round(max(res_ts), digits = 2))
      message("Min: ", round(min(res_ts), digits = 2))
    }
  }
  if (save.csv == "yes") {
    for (i in 1:length(results)) {
      namer <- names(results[i])
      write.table(as.character(namer), file = "ET_Penman.csv", 
                  dec = ".", quote = FALSE, col.names = FALSE, 
                  row.names = F, append = TRUE, sep = ",")
      write.table(data.frame(get(namer, results)), file = "ET_Penman.csv", 
                  col.names = F, append = T, sep = ",")
    }
    invisible(results)
  }
  else {
    return(results)
  }
}