library(Evapotranspiration)
library(lubridate)
library(plyr)
library(XML)
library(xml2)
library(dplyr)
library(owmr)
library(httr)
library(ggplot2)
library(jsonlite)
td <- tempdir()
source("ReadAxfBOM.r")
source("Penman2.r")



#parameters and constants

backperiod <- 8 #set period looking backward (including today) in days
futureperiod <- 4  #that is the maximum owm provides for free, so we look 4 days ahead (including today if in the morning before 15:00)

#OWM api key
Sys.setenv(OWM_API_KEY = "your owm api key") # if not set globally
cityid <- 2172517  #"Canberra" [owm city id] or name in quotes, though OWM prefers the id 

#add your ospi login data
ospi_url <- "192.168.0.1:8080" #your sprinkler ip address in your network
ospi_hash <- "324sdffs32sdfsdf" #the sprinkler hash to log into your sprinkler, you can find it via your browser or via hash convertes with your ospi password


#Your dripping refill speed.
#convert refill speed into mm/s (e.g. my system refills  ~ 20 mm per hour)
drip.speed <- 20/3600 #mm/s 

location <- "canberra_airport-" #bom location, check the ftp server for names


#download bom via ftp available ETO data from bom
ftppath <- "ftp://ftp.bom.gov.au/anon/gen/clim_data/IDCKWCDEA0/tables/nsw/canberra_airport/"


#bom axf files for recent data

#find the page for your area and copy the link for the axf file:
#e.g. www.bom.gov.au/products/IDN60903/IDN60903.94926.shtml for Canberra


download.file(url="http://www.bom.gov.au/fwo/IDN60801/IDN60801.94926.axf", destfile = file.path(td, "recent.csv"))



#find two latest file from the current and previous month(to guarantee at least one week backwards)
now <- Sys.time()

#this.month
this.month <-month(now)
this.year <- year(now)

#last.month
last.month <-this.month-1
if (last.month==0) {
  last.month <-12; last.year <-this.year-1
  } else last.year <- this.year


f1 <- download.file(url =paste0(ftppath, location,last.year,sprintf("%02d", last.month),".csv"), destfile = file.path(td,"f1.csv"))
f2 <- download.file(url =paste0(ftppath, location,this.year,sprintf("%02d", this.month),".csv"), destfile = file.path(td,"f2.csv")) 


d1 <- read.csv(file.path(td, "f1.csv"), skip = 13, header=FALSE)
d1 <- d1[-nrow(d1),]

d2 <- read.csv(file.path(td, "f2.csv"), skip = 13, header=FALSE)
d2 <- d2[-nrow(d2),]

bom <-rbind(d1,d2)

colnames(bom)<- c("station", "date","eto", "rain", "pan", "maxTemp", "minTemp", "maxRH", "minRH", "wind", "solar")
#remove duplicated rows
bom <- bom[!duplicated(bom$date),]
#remove empty lines
bom$date <- as.character(bom$date)
bom$date <- as.Date(bom$date, format = "%d/%m/%Y")



today =Sys.Date()

weather<- data.frame(date=as.Date((today-backperiod):(today+futureperiod)))

weather$history <- ifelse(weather$date<today,"past","future")
weather$history[weather$date==today] <- "presence"
weather$history <- factor(weather$history)

#fill all eto and rain what we have
weather <- join(weather, bom[,c("date", "eto","rain")])

weather$eto.source <- ifelse(!is.na(weather$eto), "bom", NA)
weather$rain.source <- ifelse(!is.na(weather$rain), "bom", NA)


#fill in the last day/current using until now using actual weather data and Eto formulas

##get recent data (last 72 hours)

#find station data from bom


#last 72 hours to fill current time slots in weather
recent <- ReadAxfBOM(file.path(td, "recent.csv"))
data("constants")
#calculate Eto for the current day
cc <- list(Elev=595, lambda=2.45, lat_rad=--0.6145653, Gsc=constants$Gsc, z=10, sigma=constants$sigma, G=0, as=constants$as, bs=constants$bs)

recent$cloud_oktas <- ifelse(recent$cloud_oktas==-9999, NA, recent$cloud_oktas)


#check time to decide if actual data or forecast should be used for today (from   4 pm we use the actual data, before we use the predicted rain data (to make sure we do not miss predicted rain in the evening...)

if (hour(Sys.time())>15) recrange <- today else recrange <- today-1


for (i in (today-1):recrange)
{
  tt <- which(weather$date==i)
if (is.na(weather$eto[tt]))
{
index <- day(recent$Timestamp)==day(as.Date(i))
dd <- list(Date.daily=as.Date(i), Tmax=max(recent$air_temp[index],na.rm=T), Tmin=min(recent$air_temp[index],na.rm=T),  J=zoo(yday(as.Date(i))),  RHmax=max(recent$rel_hum[index],na.rm=T), RHmin=min(recent$rel_hum[index],na.rm=T), uz=median(recent$wind_spd_kmh*1000/3600, na.rm = T), Cd=median(recent$cloud_oktas[index], na.rm = T))

et <-ET.Penman2(dd, cc, message="no", ts="daily",save.csv="no", solar="cloud")#, alpha=0.23, solar="data")#, z0=0.012)

weather[tt,"eto"] <- round(et$ET.Daily,2)
weather[tt, "eto.source"] <- "Penman.recent"


###rainfall today....
hours <- hour(recent$Timestamp)
rain.to9 <- max(recent$rain_trace[index & hours<=9])
#rain.from9<- max(recent$rain_trace[index & hours>9]) 

weather[tt,"rain"] <- rain.to9 
weather[tt, "rain.source"] <- "bom.recent"

}
}

#get temperature forecast from owm for today (if not yet filled in) until 
#predict eto of future using Penman2

forecast <- get_forecast(cityid, units = "metric") %>%  owmr_as_tibble()


forecast$day <- as.Date(as.POSIXct(forecast$dt, origin="1970-01-01"), tz="Australia/Sydney")
index <- which(is.na(weather$eto))

for (ii in index)
{
  foreday <- weather$date[ii]
  
  ww <- forecast[forecast$day==foreday,]
  
  dd <- list(Date.daily=foreday, Tmax=max(ww$temp,na.rm=T), Tmin=min(ww$temp),  J=zoo(yday(foreday)),  RHmax=max(ww$humidity,na.rm=T), RHmin=min(ww$humidity,na.rm=T), uz=median(ww$wind_speed, na.rm = T), Cd=median(ww$clouds_all*0.08, na.rm = T))
  
  et <-ET.Penman2(dd, cc, message="no", ts="daily",save.csv="no", solar="cloud")#, alpha=0.23, z0=0.012)  
  
  weather[ii,"eto"] <- round(et$ET.Daily,2)
  weather[ii, "eto.source"] <- "Penman.owm" 
  
  rr <- sum(ww$rain_3h,na.rm=T)
  if(is.na(rr)) rr <- 0
  weather[ii,"rain"] <- rr
  weather[ii, "rain.source"] <- "owm" 
  
}

#get logs from the backperiod from OSPI and put it into the table

ospi_api <- paste0(ospi_url,"/jl?pw=", ospi_hash, "&hist=", backperiod+2)


js <- GET(ospi_api)
logs<-content(js,as="text") 
logs.df <- data.frame(fromJSON(logs), stringsAsFactors = FALSE)

colnames(logs.df)  <- c("pid","sid","dur","end")

logs.df$enddt <- as.POSIXct(as.numeric(logs.df[,"end"]), origin="1970-01-01", tz="UTC")
tz(logs.df$enddt) <- "Australia/Sydney" 
logs.df$startdt <- logs.df$enddt-as.numeric(logs.df$dur)
logs.df$date <- as.Date(logs.df$startdt, tz="Australia/Sydney")

#only station 0


logs.df <- logs.df[logs.df$sid=="0",]

for (i in 1:nrow(weather))
{
  dd <- weather$date[i]
  index <-logs.df$date == dd
  dumdr <- sum(as.numeric(logs.df$dur[index]), na.rm=T)
  weather$driptime[i] <- dumdr
}




weather$irrigation <- round(weather$driptime * drip.speed,2)





### plot
##kc (plant specific constant, e.g. turf-0.7)
kc <- 0.7
weather$etkc <- weather$eto*kc

#irrigation effectivity (0.9 for drip irrigation)
irr.effectivity  <- 0.9

weather$irreff <- weather$irrigation*irr.effectivity

weather$water.level <- cumsum(weather$rain+weather$irreff)+-cumsum(weather$etkc)

plot(weather$date,weather$water.level , type="l", ylim=c(-20,20))
abline(h=0, lty=3)

ggplot(weather, aes(x=date, y=water.level))+geom_rect(aes(NULL, NULL, 
                xmin=date-0.5, xmax=date+0.5, ymin=min(water.level), ymax=max(water.level), fill=history), alpha=0.3)+geom_bar(stat="identity", aes(y=irrigation),fill="blue", width=0.5)   + scale_x_date(date_breaks = "days" , date_labels = "%d-%b") + scale_y_continuous(breaks=seq(-50,50,5)) + geom_bar(stat="identity", aes(y=rain), fill="purple", width=0.25)+geom_line(color="black", lwd=2) + geom_line(aes(y=eto), color="brown", lty=2, lwd=1.4)

weather

#NOw we need to decide when if to sprinkel

#what is the current status?
#if below then sprinkle threshold set to sprinkel
#otherwise wait

#what is the predicted status
#if tomorrow too low then set to sprinkle!!!!
#otherwise wait a day

#set to sprinkel ? 
#If yes, what is the predicted rain tomorrow?
#Will it help to go over threshold e.g. if rain>5 mm then wait and see 
#if not enough then sprinkel




